"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const env_1 = require("../lib/env");
const logger_1 = require("../lib/logger");
const mongodb_1 = require("../client/mongodb");
const postgres_1 = require("../client/postgres");
const mongodb_2 = require("../dao/mongodb");
const postgres_2 = require("../dao/postgres");
const service_1 = require("../service");
const new_1 = require("./command/new");
const version_1 = require("./command/version");
const list_1 = require("./command/list");
const up_1 = require("./command/up");
const down_1 = require("./command/down");
const printUsage = () => {
    console.log('Usage: migrate [new|version|list|up|down|help]');
    console.log('Environment variables:');
    console.log('  REVISIONS_NAMESPACE ' +
        '- namespace for managing more than one version');
    console.log('  REVISIONS_DIRECTORY ' +
        '- path to revisions directory containing revisions files');
    console.log('  REVISIONS_CLIENT    ' +
        '- database client type (eg. postgresql, mongodb, etc)');
};
const loadCommand = (commandName) => {
    const command = {
        new: new_1.command,
        version: version_1.command,
        list: list_1.command,
        up: up_1.command,
        down: down_1.command,
        help: printUsage
    }[commandName.toLowerCase()];
    if (command === undefined) {
        printUsage();
        throw new Error('unknown command');
    }
    return command;
};
const loadConfig = () => {
    // TODO: option to load from file
    const config = {
        revisionsNamespace: (0, env_1.expectEnv)('REVISIONS_NAMESPACE'),
        revisionsDirectory: (0, env_1.expectEnv)('REVISIONS_DIRECTORY'),
        clientModule: (0, env_1.expectEnv)('REVISIONS_CLIENT')
    };
    return config;
};
const main = () => __awaiter(void 0, void 0, void 0, function* () {
    const args = process.argv.slice(2);
    if (args.length === 0) {
        printUsage();
        throw new Error('command required');
    }
    const config = loadConfig();
    const command = loadCommand(args[0]);
    const logger = (0, logger_1.getLogger)('CLI');
    // Load connection manager and persistent facade
    let db;
    let dao;
    switch (config.clientModule) {
        case 'mongodb':
            {
                const uri = (0, env_1.expectEnv)('MONGODB_URI');
                const connection = mongodb_1.MongoDBConnectionManager.createConnection(uri);
                db = new mongodb_1.MongoDBConnectionManager(connection, { logger });
                dao = new mongodb_2.MongoDBPersistence({ logger });
            }
            break;
        case 'postgresql':
            {
                const pool = postgres_1.PostgreSQLConnectionManager.createPool();
                db = new postgres_1.PostgreSQLConnectionManager(pool, { logger });
                dao = new postgres_2.PostgreSQLPersistence({ logger });
            }
            break;
        default: {
            throw new Error(`unknown client ${config.clientModule}`);
        }
    }
    process.on('exit', () => {
        void db.shutdown();
    });
    const service = new service_1.MigrationService({ dao, logger });
    yield command(config, db, dao, service, ...args);
});
const onSuccess = () => {
    process.exit(0);
};
const onFailure = (reason) => {
    if (reason !== undefined) {
        console.error(reason);
    }
    process.exit(1);
};
main().then(onSuccess, onFailure);
