"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgreSQLConnectionManager = void 0;
const fs_1 = require("fs");
const pg_1 = require("pg");
const env_1 = require("../lib/env");
const logger_1 = require("../lib/logger");
class PostgreSQLConnectionManager {
    constructor(pool, options) {
        var _a;
        this.pool = pool;
        this.logger = (_a = options === null || options === void 0 ? void 0 : options.logger) !== null && _a !== void 0 ? _a : (0, logger_1.getLogger)(PostgreSQLConnectionManager.name);
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug('PostgreSQL client is connecting');
            return yield this.pool.connect();
        });
    }
    disconnect(client) {
        this.logger.debug('PostgreSQL client is disconnecting');
        client.release();
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.warn('PostgreSQL client is shutting down');
            yield this.pool.end();
        });
    }
    ping() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug('PostgreSQL client is pinging server');
            yield this.pool.query('SELECT');
        });
    }
    transaction(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            // Reference: https://node-postgres.com/features/transactions
            const client = yield this.connect();
            try {
                this.logger.debug('Beginning transaction');
                yield client.query('BEGIN');
                this.logger.debug('Running transaction callback');
                const result = yield callback(client);
                this.logger.debug('Committing transaction');
                yield client.query('COMMIT');
                this.logger.debug('Committed transaction');
                return result;
            }
            catch (error) {
                this.logger.warn('Rolling back transaction');
                yield client.query('ROLLBACK');
                throw error;
            }
            finally {
                this.disconnect(client);
            }
        });
    }
    static canSsl() {
        return (process.env.PGSSLROOTCERT !== undefined &&
            process.env.PGSSLKEY !== undefined &&
            process.env.PGSSLCERT !== undefined);
    }
    static createPoolConfig(config) {
        return Object.assign({ user: (0, env_1.expectEnv)('PGUSER'), password: (0, env_1.expectEnv)('PGPASSWORD'), database: (0, env_1.expectEnv)('PGDATABASE'), host: (0, env_1.expectEnv)('PGHOST'), port: parseInt((0, env_1.expectEnv)('PGPORT')), 
            // ...
            max: 5 }, config);
    }
    static createSslPoolConfig(config) {
        return Object.assign(Object.assign(Object.assign({}, this.createPoolConfig()), { ssl: {
                rejectUnauthorized: false,
                ca: (0, fs_1.readFileSync)((0, env_1.expectEnv)('PGSSLROOTCERT')).toString(),
                key: (0, fs_1.readFileSync)((0, env_1.expectEnv)('PGSSLKEY')).toString(),
                cert: (0, fs_1.readFileSync)((0, env_1.expectEnv)('PGSSLCERT')).toString()
            } }), config);
    }
    static createPool(config) {
        if (this.canSsl()) {
            return new pg_1.Pool(this.createSslPoolConfig(config));
        }
        else {
            return new pg_1.Pool(this.createPoolConfig(config));
        }
    }
}
exports.PostgreSQLConnectionManager = PostgreSQLConnectionManager;
