"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoDBPersistence = exports.MigrationsSchema = exports.MigrationsDefinition = exports.PROPERTY_NAME_UPDATED_AT = exports.PROPERTY_NAME_CREATED_AT = exports.PROPERTY_NAME_FILE = exports.PROPERTY_NAME_VERSION = exports.PROPERTY_NAME_PREVIOUS_VERSION = exports.PROPERTY_NAME_NAMESPACE = exports.COLLECTION_NAME = void 0;
const console_1 = require("console");
const mongoose_1 = require("mongoose");
const logger_1 = require("../lib/logger");
const service_1 = require("../service");
exports.COLLECTION_NAME = 'migrations';
// read and write
exports.PROPERTY_NAME_NAMESPACE = 'namespace';
exports.PROPERTY_NAME_PREVIOUS_VERSION = 'previousVersion';
exports.PROPERTY_NAME_VERSION = 'version';
exports.PROPERTY_NAME_FILE = 'file';
// read only
exports.PROPERTY_NAME_CREATED_AT = 'createdAt';
exports.PROPERTY_NAME_UPDATED_AT = 'updatedAt';
exports.MigrationsDefinition = {
    [exports.PROPERTY_NAME_NAMESPACE]: {
        type: mongoose_1.Schema.Types.String,
        required: true,
        unique: true
    },
    [exports.PROPERTY_NAME_PREVIOUS_VERSION]: {
        type: mongoose_1.Schema.Types.String,
        required: false,
        unique: true
    },
    [exports.PROPERTY_NAME_VERSION]: {
        type: mongoose_1.Schema.Types.String,
        required: true
    },
    [exports.PROPERTY_NAME_FILE]: {
        type: mongoose_1.Schema.Types.String,
        required: true
    },
    [exports.PROPERTY_NAME_CREATED_AT]: {
        type: mongoose_1.Schema.Types.Date,
        required: true
    },
    [exports.PROPERTY_NAME_UPDATED_AT]: {
        type: mongoose_1.Schema.Types.Date,
        required: true
    }
};
exports.MigrationsSchema = new mongoose_1.Schema(exports.MigrationsDefinition, {
    timestamps: {
        createdAt: exports.PROPERTY_NAME_CREATED_AT,
        updatedAt: exports.PROPERTY_NAME_UPDATED_AT
    }
});
class MongoDBPersistence {
    createMigrationsModel(connection) {
        return connection.model(exports.COLLECTION_NAME, exports.MigrationsSchema);
    }
    constructor(options) {
        var _a;
        this.logger = (_a = options.logger) !== null && _a !== void 0 ? _a : (0, logger_1.getLogger)(MongoDBPersistence.name);
    }
    initialize(client) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Initialize MongoDB persistence (no-op)');
        });
    }
    acquireExclusiveLock(client) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: locking a collection is possible,
            // but not currently an available MongoDB feature
            this.logger.info('Locking MongoDB collection (no-op)');
        });
    }
    releaseExclusiveLock(client) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO
            this.logger.info('Unlocking MongoDB collection (no-op)');
        });
    }
    fetchCurrentRevision(client, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Get MongoDB revision', { namespace });
            const MigrationsModel = this.createMigrationsModel(client.connection);
            const documents = yield MigrationsModel
                .find().limit(1).lean().session(client.session);
            switch (documents.length) {
                case 0: {
                    this.logger.info('Get MongoDB revision - no revision found');
                    return;
                }
                case 1: {
                    const { previousVersion, version, file, createdAt, updatedAt } = documents[0];
                    const revision = {
                        previousVersion: previousVersion !== null && previousVersion !== void 0 ? previousVersion : undefined,
                        version,
                        file,
                        createdAt,
                        updatedAt
                    };
                    this.logger.info('Get PostgreSQL revision - revision found', { revision });
                    return revision;
                }
                default: {
                    throw new service_1.MigrationServiceError('MongoDB migration service found multiple revisions with the ' +
                        'same namespace');
                }
            }
        });
    }
    setCurrentRevision(client, namespace, revision) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Set MongoDB revision', { namespace, revision });
            (0, console_1.assert)(client.session.inTransaction());
            const MigrationsModel = this.createMigrationsModel(client.connection);
            const document = yield MigrationsModel.findOneAndUpdate({
                [exports.PROPERTY_NAME_NAMESPACE]: namespace
            }, {
                [exports.PROPERTY_NAME_NAMESPACE]: namespace,
                [exports.PROPERTY_NAME_PREVIOUS_VERSION]: (_a = revision.previousVersion) !== null && _a !== void 0 ? _a : null,
                [exports.PROPERTY_NAME_VERSION]: revision.version,
                [exports.PROPERTY_NAME_FILE]: revision.file
            }, {
                new: true,
                upsert: true,
                session: client.session
            });
            yield (document === null || document === void 0 ? void 0 : document.save());
        });
    }
    removeNamespace(client, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            (0, console_1.assert)(client.session.inTransaction());
            const MigrationsModel = this.createMigrationsModel(client.connection);
            yield MigrationsModel.findOneAndRemove({
                [exports.PROPERTY_NAME_NAMESPACE]: namespace
            }).session(client.session);
        });
    }
}
exports.MongoDBPersistence = MongoDBPersistence;
