"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgreSQLPersistence = void 0;
const logger_1 = require("../lib/logger");
const service_1 = require("../service");
const TABLE_NAME_MIGRATIONS = 'migrations';
// read and write
const COLUMN_NAME_NAMESPACE = 'namespace';
const COLUMN_NAME_PREVIOUS_VERSION = 'previous_version';
const COLUMN_NAME_VERSION = 'version';
const COLUMN_NAME_FILE = 'file';
// read only
const COLUMN_NAME_CREATED_AT = 'created_at';
const COLUMN_NAME_UPDATED_AT = 'updated_at';
class PostgreSQLPersistence {
    constructor(options) {
        var _a;
        this.logger = (_a = options.logger) !== null && _a !== void 0 ? _a : (0, logger_1.getLogger)(PostgreSQLPersistence.name);
    }
    initialize(client) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Initialize PostgreSQL persistence');
            yield client.query(`
      CREATE TABLE IF NOT EXISTS ${TABLE_NAME_MIGRATIONS} (
        ${COLUMN_NAME_NAMESPACE} TEXT UNIQUE NOT NULL,
        ${COLUMN_NAME_PREVIOUS_VERSION} TEXT UNIQUE,
        ${COLUMN_NAME_VERSION} TEXT UNIQUE NOT NULL,
        ${COLUMN_NAME_FILE} TEXT NOT NULL,
        ${COLUMN_NAME_CREATED_AT}
          TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        ${COLUMN_NAME_UPDATED_AT}
          TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP
      )
    `);
        });
    }
    acquireExclusiveLock(client) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Locking PostgreSQL table');
            yield client.query(`
      LOCK TABLE ONLY ${TABLE_NAME_MIGRATIONS}
      IN EXCLUSIVE MODE
    `);
        });
    }
    releaseExclusiveLock(client) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Unlocking PostgreSQL table (no-op)');
            // Released at end of transaction
        });
    }
    fetchCurrentRevision(client, namespace) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Get PostgreSQL revision', { namespace });
            const statement = `
      SELECT *
      FROM ${TABLE_NAME_MIGRATIONS}
      WHERE ${COLUMN_NAME_NAMESPACE} = $1
    `;
            this.logger.debug('Get PostgreSQL revision - query statement', { statement });
            const result = yield client.query(statement, [namespace]);
            switch (result.rowCount) {
                case 0: {
                    this.logger.info('Get PostgreSQL revision - no revision found');
                    break;
                }
                case 1: {
                    const revision = {
                        previousVersion: (_a = result.rows[0][COLUMN_NAME_PREVIOUS_VERSION]) !== null && _a !== void 0 ? _a : undefined,
                        version: result.rows[0][COLUMN_NAME_VERSION],
                        file: result.rows[0][COLUMN_NAME_FILE],
                        createdAt: result.rows[0][COLUMN_NAME_CREATED_AT],
                        updatedAt: result.rows[0][COLUMN_NAME_UPDATED_AT]
                    };
                    this.logger.info('Get PostgreSQL revision - revision found', { revision });
                    return revision;
                }
                default: {
                    throw new service_1.MigrationServiceError('PostgreSQL migration service found multiple revisions with the ' +
                        'same namespace');
                }
            }
        });
    }
    setCurrentRevision(client, namespace, revision) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: find out why parameterized queries do not insert file value
            this.logger.info('Set PostgreSQL revision', { namespace, revision });
            const updateStatement = `
      UPDATE ${TABLE_NAME_MIGRATIONS} SET
        ${COLUMN_NAME_PREVIOUS_VERSION} = ${revision.previousVersion === undefined
                ? 'NULL'
                : '\'' + revision.previousVersion + '\''},
        ${COLUMN_NAME_VERSION} = '${revision.version}',
        ${COLUMN_NAME_FILE} = '${revision.file}',
        ${COLUMN_NAME_UPDATED_AT} = NOW()
      WHERE ${COLUMN_NAME_NAMESPACE} = '${namespace}'
    `;
            this.logger.debug('Set PostgreSQL revision - query update statement', { updateStatement });
            yield client.query(updateStatement);
            const insertStatement = `
      INSERT INTO ${TABLE_NAME_MIGRATIONS} (
        ${COLUMN_NAME_NAMESPACE},
        ${COLUMN_NAME_PREVIOUS_VERSION},
        ${COLUMN_NAME_VERSION},
        ${COLUMN_NAME_FILE}
      ) VALUES (
        '${namespace}',
        ${revision.previousVersion === undefined
                ? 'NULL'
                : '\'' + revision.previousVersion + '\''},
        '${revision.version}',
        '${revision.file}'
      )
      ON CONFLICT (${COLUMN_NAME_NAMESPACE}) DO NOTHING
    `;
            this.logger.debug('Set PostgreSQL revision - query insert statement', { insertStatement });
            yield client.query(insertStatement);
        });
    }
    removeNamespace(client, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('Remove namespace', { namespace });
            yield client.query(`
      DELETE FROM ${TABLE_NAME_MIGRATIONS}
      WHERE ${COLUMN_NAME_NAMESPACE} = $1
    `, [namespace]);
        });
    }
}
exports.PostgreSQLPersistence = PostgreSQLPersistence;
