"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFileContent = exports.generateFileName = exports.resolveDowngradePath = exports.resolveUpgradePath = exports.sortAndVerifyRevisionModules = exports.verifyRevisionModule = exports.loadDirectory = exports.loadFile = exports.fileFilter = exports.computeVersion = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const hash_1 = require("./lib/hash");
const computeVersion = (previousVersion, fileHash) => {
    // Build a Merkel Tree
    // https://en.wikipedia.org/wiki/Merkle_tree
    return previousVersion === undefined
        ? fileHash
        : (0, hash_1.hash)(fileHash + previousVersion);
};
exports.computeVersion = computeVersion;
const fileFilter = (filename) => {
    const pattern = /^.+\.revision\.(ts|js)$/;
    return pattern.test(filename);
};
exports.fileFilter = fileFilter;
const loadFile = (file) => {
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    const { previousVersion, up, down } = require(file);
    if (previousVersion !== undefined && typeof previousVersion !== 'string') {
        throw new Error('revision file missing previousVersion variable');
    }
    if (typeof up !== 'function') {
        throw new Error('revision file missing up function');
    }
    if (typeof down !== 'function') {
        throw new Error('revision file missing down function');
    }
    const fileHash = (0, hash_1.hashFile)(file);
    return {
        up,
        down,
        previousVersion,
        version: (0, exports.computeVersion)(previousVersion, fileHash),
        file,
        fileHash
    };
};
exports.loadFile = loadFile;
const loadDirectory = (revisionDirectory) => {
    // assume revision order is lexicographically ordered
    const files = (0, fs_1.readdirSync)(revisionDirectory)
        .filter(exports.fileFilter)
        .map((file) => (0, path_1.join)(revisionDirectory, file));
    const revisionModules = [];
    for (const file of files) {
        revisionModules.push((0, exports.loadFile)(file));
    }
    if (revisionModules.length === 0) {
        return [];
    }
    else {
        return (0, exports.sortAndVerifyRevisionModules)(revisionModules);
    }
};
exports.loadDirectory = loadDirectory;
const verifyRevisionModule = (trustedRevision, untrustedRevisionModule) => {
    var _a, _b;
    const hasValidPreviousVersion = trustedRevision.previousVersion === untrustedRevisionModule.previousVersion;
    if (!hasValidPreviousVersion) {
        throw new Error('previous version invalid - ' +
            `expected ${(_a = trustedRevision.previousVersion) !== null && _a !== void 0 ? _a : 'undefined (base)'} ` +
            `but got ${(_b = untrustedRevisionModule.previousVersion) !== null && _b !== void 0 ? _b : 'undefined (base)'}`);
    }
    const hasValidVersion = trustedRevision.version === untrustedRevisionModule.version;
    if (!hasValidVersion) {
        throw new Error('version mismatch - ' +
            `expected ${trustedRevision.version} ` +
            `but got ${untrustedRevisionModule.version} ` +
            `for file ${trustedRevision.file}`);
    }
    return untrustedRevisionModule;
};
exports.verifyRevisionModule = verifyRevisionModule;
const sortAndVerifyRevisionModules = (unsortedRevisionModules) => {
    // complexity: runtime is O(n), space O(n)
    // determine initial revision and build lookup table...
    let initialRevisionModule;
    const revisionModuleByPreviousVersion = {};
    for (const revisionModule of unsortedRevisionModules) {
        if (revisionModule.previousVersion === undefined) {
            initialRevisionModule = revisionModule;
        }
        else {
            revisionModuleByPreviousVersion[revisionModule.previousVersion] = revisionModule;
        }
    }
    if (initialRevisionModule === undefined) {
        throw new Error('missing first revision');
    }
    // use lookup table to traverse from initial revision to final revision
    const sortedRevisionModules = [];
    let nextRevisionModule = initialRevisionModule;
    while (true) {
        sortedRevisionModules.push(nextRevisionModule);
        nextRevisionModule = revisionModuleByPreviousVersion[nextRevisionModule.version];
        if (nextRevisionModule === undefined) {
            break;
        }
    }
    if (sortedRevisionModules.length !== unsortedRevisionModules.length) {
        const suspectedFile = sortedRevisionModules[sortedRevisionModules.length - 1].file;
        throw new Error(`revisions are disjoint - check file ${suspectedFile}`);
    }
    return sortedRevisionModules;
};
exports.sortAndVerifyRevisionModules = sortAndVerifyRevisionModules;
const resolveUpgradePath = (revisionModules, // local file system (sorted and verified)
currentRevision // remote server (source of truth)
) => {
    /* scenarios:
     * - nothing to upgrade
     * - all revisions are pending
     * - some pending revisions
     */
    const sortedRevisionModules = (0, exports.sortAndVerifyRevisionModules)(revisionModules);
    // locate next pending revision...
    let nextRevisionModuleIndex;
    if (currentRevision === undefined) {
        // all revisions are pending
        // locate first pending revision module
        if (sortedRevisionModules.length === 0) {
            // nothing to do
            return {
                initialRevision: undefined,
                pendingRevisionModules: []
            };
        }
        nextRevisionModuleIndex = 0;
    }
    else {
        // locate and verify revision module
        const currentRevisionModuleIndex = sortedRevisionModules.findIndex((revisionModule) => revisionModule.file === currentRevision.file);
        if (currentRevisionModuleIndex < 0) {
            throw new Error('no upgrade path - cannot find revision module for current version');
        }
        const currentRevisionModule = sortedRevisionModules[currentRevisionModuleIndex];
        (0, exports.verifyRevisionModule)(currentRevision, currentRevisionModule);
        nextRevisionModuleIndex = currentRevisionModuleIndex + 1;
        if (nextRevisionModuleIndex >= sortedRevisionModules.length) {
            // nothing to do
            return {
                initialRevision: currentRevision,
                pendingRevisionModules: []
            };
        }
    }
    return {
        initialRevision: currentRevision,
        pendingRevisionModules: sortedRevisionModules.slice(nextRevisionModuleIndex)
    };
};
exports.resolveUpgradePath = resolveUpgradePath;
const resolveDowngradePath = (revisionModules, currentRevision) => {
    /* scenarios:
     * - nothing to downgrade
     * - has no previous version / previous version is base
     * - has previous version
     */
    if (currentRevision === undefined) {
        // nothing to downgrade
        return {
            finalRevision: undefined,
            pendingRevisionModules: []
        };
    }
    const currentRevisionModule = revisionModules.find((revisionModule) => revisionModule.file === currentRevision.file);
    if (currentRevisionModule === undefined) {
        throw new Error('current revision module not found');
    }
    (0, exports.verifyRevisionModule)(currentRevision, currentRevisionModule);
    if (currentRevisionModule.previousVersion === undefined) {
        // first revision after base
        return {
            finalRevision: undefined,
            pendingRevisionModules: [currentRevisionModule]
        };
    }
    const previousAppliedRevisionModule = revisionModules.find((revisionModule) => revisionModule.version === currentRevisionModule.previousVersion);
    if (previousAppliedRevisionModule === undefined) {
        // should have a previous version
        throw new Error('no upgrade path - missing revision dependency');
    }
    return {
        finalRevision: {
            file: previousAppliedRevisionModule.file,
            version: previousAppliedRevisionModule.version,
            previousVersion: previousAppliedRevisionModule.previousVersion
        },
        pendingRevisionModules: [currentRevisionModule]
    };
};
exports.resolveDowngradePath = resolveDowngradePath;
const generateFileName = (description) => {
    const timestamp = Date.now();
    const label = description
        .toLowerCase()
        .replace(/[^a-z0-9]/g, '-')
        .substring(0, 50);
    return `${timestamp}_${label}.revision.js`;
};
exports.generateFileName = generateFileName;
const generateFileContent = (previousVersion) => ((previousVersion === undefined
    ? 'const previousVersion = undefined\n'
    : `const previousVersion = '${previousVersion}'\n`) +
    'const up = async (client) => {}\n' +
    'const down = async (client) => {}\n' +
    'module.exports = { previousVersion, up, down }\n');
exports.generateFileContent = generateFileContent;
