"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrationService = exports.MigrationServiceError = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const logger_1 = require("../lib/logger");
const revision_1 = require("../revision");
class MigrationServiceError extends Error {
}
exports.MigrationServiceError = MigrationServiceError;
class MigrationService {
    constructor(options) {
        var _a;
        this.dao = options.dao;
        this.logger = (_a = options.logger) !== null && _a !== void 0 ? _a : (0, logger_1.getLogger)(MigrationService.name);
    }
    fetchCurrentRevision(client, request) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.dao.fetchCurrentRevision(client, request.namespace);
        });
    }
    newRevision(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const untrustedRevisionModules = (0, revision_1.loadDirectory)(request.directory);
            let latestVersion;
            if (untrustedRevisionModules.length > 0) {
                const { pendingRevisionModules } = (0, revision_1.resolveUpgradePath)(untrustedRevisionModules, undefined);
                const latestRevisionModule = pendingRevisionModules[pendingRevisionModules.length - 1];
                latestVersion = latestRevisionModule.version;
            }
            const filePath = (0, path_1.join)(request.directory, (0, revision_1.generateFileName)(request.description));
            const fileContent = (0, revision_1.generateFileContent)(latestVersion);
            (0, fs_1.writeFileSync)(filePath, fileContent);
            return filePath;
        });
    }
    upgrade(client, request) {
        return __awaiter(this, void 0, void 0, function* () {
            // Upgrade to latest version - apply zero or more pending revisions
            /* IMPORTANT
             *
             * NodeJS fs.readdirSync() returns files lexicographically, which is NOT
             * the same order that revisions will be applied.
             *
             * To ensure a rollback of all pending revisions, pass a database
             * transaction and lock the table in exclusive mode to prevent concurrent
             * writes.
             */
            this.logger.info('Upgrade database', { request });
            const revisionModules = (0, revision_1.loadDirectory)(request.directory);
            const currentRevision = yield this.fetchCurrentRevision(client, {
                namespace: request.namespace
            });
            const upgradePath = (0, revision_1.resolveUpgradePath)(revisionModules, currentRevision);
            const { pendingRevisionModules } = upgradePath;
            if (pendingRevisionModules.length === 0) {
                this.logger.info('No pending revisions');
                return upgradePath;
            }
            for (const pendingRevisionModule of pendingRevisionModules) {
                this.logger.info('Applying revision', { revisionModule: pendingRevisionModule });
                const { up } = pendingRevisionModule;
                yield up(client);
            }
            if (pendingRevisionModules.length > 0) {
                const finalRevision = pendingRevisionModules[pendingRevisionModules.length - 1];
                yield this.dao.setCurrentRevision(client, request.namespace, finalRevision);
            }
            return upgradePath;
        });
    }
    downgrade(client, request) {
        return __awaiter(this, void 0, void 0, function* () {
            // Downgrade from current version - revert only one revision
            this.logger.info('Downgrade database', { request });
            const revisionModules = (0, revision_1.loadDirectory)(request.directory);
            const currentRevision = yield this.fetchCurrentRevision(client, {
                namespace: request.namespace
            });
            const downgradePath = (0, revision_1.resolveDowngradePath)(revisionModules, currentRevision);
            const { finalRevision, pendingRevisionModules } = downgradePath;
            if (pendingRevisionModules.length === 0) {
                this.logger.debug('Nothing to downgrade');
                return downgradePath;
            }
            const { down } = pendingRevisionModules[0];
            yield down(client);
            if (finalRevision === undefined) {
                yield this.dao.removeNamespace(client, request.namespace);
                return downgradePath;
            }
            yield this.dao.setCurrentRevision(client, request.namespace, finalRevision);
            return downgradePath;
        });
    }
}
exports.MigrationService = MigrationService;
